#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#

import os
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

DEBUG = False