#
# PygamePal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#

import pygame

class SpriteTextureList():

    '''
    Used for storing a list of textures (and other info) for a SpriteImage state.
    (Users of the PygamePal library should not need to use this class directly. Instead, it is used by pygamepal.SpriteImage).
    '''

    def __init__(self):
        self._textures = []
        self._animationDelay = 12
        self._loop = True
        self._hFlip = False
        self._vFlip = False
        self._offset = (0, 0)